using SymbolicIntegration, SymbolicUtils
@syms x

#Integration Test Problems from 
#https://rulebasedintegration.org/testProblems.html

#1 Algebraic functions\1.3 Miscellaneous\1.3.1 Rational functions.input
#Problems from Calculus textbooks and competitions
problems = [
#Ayres Calculus, 1964 edition
[(3*x-4*x^2+3*x^3)//(1+x^2),x,4,"-4*x+3/2*x^2+4*atan(x)"],
[(5+3*x)//(1-x-x^2+x^3),x,3,"4/(1-x)+atanh(x)"],
[(-1-x-x^3+x^4)//(-x^2+x^3),x,3,"(-1)/x+1/2*x^2-2*log(1-x)+2*log(x)"],
[(2+x+x^2+x^3)//(2+3*x^2+x^4),x,6,"atan(x)+1/2*log(2+x^2)"],
[(-4+8*x-4*x^2+4*x^3-x^4+x^5)//(2+x^2)^3,x,5,"(-1)/(2+x^2)^2+1/2*log(2+x^2)-atan(x/sqrt(2))/sqrt(2)"],
[(-1-3*x+x^2)//(-2*x+x^2+x^3),x,3,"-log(1-x)+1/2*log(x)+3/2*log(2+x)"],
[(3-x+3*x^2-2*x^3+x^4)//(3*x-2*x^2+x^3),x,4,"1/2*x^2+log(x)-1/2*log(3-2*x+x^2)"],
[(-1+x+x^3)//(1+x^2)^2,x,4,"-1/2*x/(1+x^2)-1/2*atan(x)+1/2*log(1+x^2)"],
[(1+2*x-x^2+8*x^3+x^4)//((x+x^2)*(1+x^3)),x,7,"(-3)/(1+x)+log(x)-2*log(1+x)+log(1-x+x^2)-2*atan((1-2*x)/sqrt(3))/sqrt(3)"],
[(15-5*x+x^2+x^3)//((5+x^2)*(3+2*x+x^2)),x,7,"1/2*log(3+2*x+x^2)+5*atan((1+x)/sqrt(2))/sqrt(2)-atan(x/sqrt(5))*sqrt(5)"],
[(-3+25*x+23*x^2+32*x^3+15*x^4+7*x^5+x^6)//((1+x^2)^2*(2+x+x^2)^2),x,6,"(-3)/(1+x^2)+1/(2+x+x^2)+log(1+x^2)-log(2+x+x^2)"],

#Edwards and Penney Calculus
[1//((1+x^2)*(4+x^2)),x,3,"-1/6*atan(1/2*x)+1/3*atan(x)"],
#[(a+b*x^3)//(1+x^2),x,5,"1/2*b*x^2+a*atan(x)-1/2*b*log(1+x^2)"],
[(x+x^2)//((4+x)*(-4+x^2)),x,4,"-1/2*atanh(1/2*x)+log(4+x)"],
[(4+x^2)//((1+x^2)*(2+x^2)),x,3,"3*atan(x)-atan(x/sqrt(2))*sqrt(2)"],
[(5-4*x+3*x^2+x^4)//((-1+x)^2*(1+x^2)),x,5,"5/2/(1-x)+x+2*atan(x)+1/2*log(1-x)+3/4*log(1+x^2)"],
[(1+x^4)//(2+x^2),x,3,"-2*x+1/3*x^3+5*atan(x/sqrt(2))/sqrt(2)"],
[(2+2*x+x^4)//(x^4+x^5),x,3,"(-2/3)/x^3+log(1+x)"],
[(-1-5*x+2*x^2)//(2-x-2*x^2+x^3),x,2,"2*log(1-x)-log(2-x)+log(1+x)"],
[(2+x+x^3)//(1+2*x^2+x^4),x,5,"x/(1+x^2)+atan(x)+1/2*log(1+x^2)"],
[(1+2*x+x^2+x^3)//(1+2*x^2+x^4),x,5,"(-1/2)/(1+x^2)+atan(x)+1/2*log(1+x^2)"],
[(3+4*x)//((1+x^2)*(2+x^2)),x,8,"3*atan(x)+2*log(1+x^2)-2*log(2+x^2)-3*atan(x/sqrt(2))/sqrt(2)"],
[(2+x)//((1+x^2)*(4+x^2)),x,8,"-1/3*atan(1/2*x)+2/3*atan(x)+1/6*log(1+x^2)-1/6*log(4+x^2)"],

#Grossman Calculus
[(2-x+x^3)//(-7-6*x+x^2),x,5,"6*x+1/2*x^2+169/4*log(7-x)-1/4*log(1+x)"],
[(-1+x^5)//(-1+x^2),x,4,"1/2*x^2+1/4*x^4+log(1+x)"],
[(5+2*x-x^2+x^3)//(1+x+x^2),x,6,"-2*x+1/2*x^2+3/2*log(1+x+x^2)+11*atan((1+2*x)/sqrt(3))/sqrt(3)"],
[(-3+x-2*x^3+x^4)//(10-8*x+2*x^2),x,6,"3/2*x+1/2*x^2+1/6*x^3+6*atan(2-x)+3/4*log(5-4*x+x^2)"],
[(1+2*x+3*x^2+x^3)//((-3+x)*(-2+x)*(-1+x)),x,2,"x+7/2*log(1-x)-25*log(2-x)+61/2*log(3-x)"],
[(2-7*x+x^2-x^3+x^4)//(-24-14*x+x^2+x^3),x,2,"-2*x+1/2*x^2+13/3*log(4-x)-22/3*log(2+x)+20*log(3+x)"],
[(2+x^2)//((-1+x)^2*x*(1+x)),x,2,"3/2/(1-x)-5/4*log(1-x)+2*log(x)-3/4*log(1+x)"],
[(3+x^2+x^3)//(2+x^2)^2,x,4,"1/4*(4+x)/(2+x^2)+1/2*log(2+x^2)+5/4*atan(x/sqrt(2))/sqrt(2)"],
[(-35+70*x-4*x^2+2*x^3)//((26-10*x+x^2)*(17-2*x+x^2)),x,10,"-15033/1025*atan(5-x)-4607/4100*atan(1/4*(-1+x))+1003/1025*log(26-10*x+x^2)+22/1025*log(17-2*x+x^2)"],
[(2+x^2)//((-5+x)*(-3+x)*(4+x)),x,2,"-11/14*log(3-x)+3/2*log(5-x)+2/7*log(4+x)"],
[x^4//((-1+x)*(2+x^2)),x,5,"x+1/2*x^2+1/3*log(1-x)-2/3*log(2+x^2)-2/3*atan(x/sqrt(2))*sqrt(2)"],

#Spivak Calculus
[(-1+7*x+2*x^2)//(-1-x+x^2+x^3),x,2,"(-3)/(1+x)+2*log(1-x)"],
[(1+2*x)//(-1+3*x-3*x^2+x^3),x,2,"(-3/2)/(1-x)^2+2/(1-x)"],
[(5-5*x+7*x^2+x^3)//((-1+x)^2*(1+x)^3),x,2,"1/(1-x)+(-2)/(1+x)^2"],
[(1+3*x+3*x^2)//(1+2*x+2*x^2+x^3),x,6,"log(1+x)+log(1+x+x^2)-2*atan((1+2*x)/sqrt(3))/sqrt(3)"],

#Stewart Calculus
[(-1+2*x+x^2)//(-2*x+3*x^2+2*x^3),x,3,"1/10*log(1-2*x)+1/2*log(x)-1/10*log(2+x)"],
[(1+4*x-2*x^2+x^4)//(1-x-x^2+x^3),x,2,"2/(1-x)+x+1/2*x^2+log(1-x)-log(1+x)"],
[(4-x+2*x^2)//(4*x+x^3),x,6,"-1/2*atan(1/2*x)+log(x)+1/2*log(4+x^2)"],
[(1+x^2+x^3)//((-1+x)*x*(1+x^2)^3*(1+x+x^2)),x,14,"1/8*(1+x)/(1+x^2)^2-3/8*(1-x)/(1+x^2)+3/16*x/(1+x^2)+7/16*atan(x)+1/8*log(1-x)-log(x)+15/16*log(1+x^2)-1/2*log(1+x+x^2)-atan((1+2*x)/sqrt(3))/sqrt(3)"],
[(1-3*x+2*x^2-x^3)//(1+x^2)^2,x,4,"1/2*(2-x)/(1+x^2)+3/2*atan(x)-1/2*log(1+x^2)"],
[(1-3*x+2*x^2-x^3)//(x*(1+x^2)^2),x,6,"1/2*(-1-2*x)/(1+x^2)-2*atan(x)+log(x)-1/2*log(1+x^2)"],
[(1-x-x^2+x^3+x^4)//(-x+x^3),x,4,"x+1/2*x^2-log(x)+1/2*log(1-x^2)"],
[(2-4*x^2+x^3)//((1+x^2)*(2+x^2)),x,8,"6*atan(x)-1/2*log(1+x^2)+log(2+x^2)-5*atan(x/sqrt(2))*sqrt(2)"],
[(1+x^2+x^4)//((1+x^2)*(4+x^2)^2),x,6,"-13/24*x/(4+x^2)+25/144*atan(1/2*x)+1/9*atan(x)"],
[(1+x^2+x^3)//(2*x^2+x^3+x^4),x,7,"(-1/2)/x-1/4*log(x)+5/8*log(2+x+x^2)+1/4*atan((1+2*x)/sqrt(7))/sqrt(7)"],
[(1-12*x+x^2+x^3)//(-12+x+x^2),x,5,"1/2*x^2-2/7*atanh(1/7*(1+2*x)),1/2*x^2+1/7*log(3-x)-1/7*log(4+x)"],
[(-3+5*x+6*x^2)//(-3*x+2*x^2+x^3),x,3,"2*log(1-x)+log(x)+3*log(3+x)"],
[(-2+3*x+5*x^2)//(2*x^2+x^3),x,3,"1/x+2*log(x)+3*log(2+x)"],
[(18-2*x-4*x^2)//(-6+x+4*x^2+x^3),x,2,"log(1-x)-2*log(2+x)-3*log(3+x)"],
[(1+x-2*x^2+x^3)//(4+5*x^2+x^4),x,7,"-3/2*atan(1/2*x)+atan(x)+1/2*log(4+x^2)"],
[(-32+5*x-27*x^2+4*x^3)//(-70-299*x-286*x^2+50*x^3-13*x^4+30*x^5),x,6,"-3146/80155*log(7-3*x)-334/323*log(1+2*x)+4822/4879*log(2+5*x)+11049/260015*log(5+x+x^2)+3988/13685*atan((1+2*x)/sqrt(19))/sqrt(19)"],
[(8-13*x^2-7*x^3+12*x^5)//(4-20*x+41*x^2-80*x^3+116*x^4-80*x^5+100*x^6),x,7,"5828/9075/(2-5*x)+1/1452*(-313-502*x)/(1+2*x^2)-59096/99825*log(2-5*x)+2843/7986*log(1+2*x^2)+503/7986*atan(x*sqrt(2))/sqrt(2),5828/9075/(2-5*x)+1/1452*(-313-502*x)/(1+2*x^2)-59096/99825*log(2-5*x)+2843/7986*log(1+2*x^2)-251/726*atan(x*sqrt(2))/sqrt(2)+272/1331*atan(x*sqrt(2))*sqrt(2)"],

#Thomas Calculus, 8th Edition
[(9+x^4)//(x^2*(9+x^2)),x,3,"(-1)/x+x-10/3*atan(1/3*x)"],
[(2*x+x^4)//(1+x^2),x,6,"-x+1/3*x^3+atan(x)+log(1+x^2)"],
[(-x+x^3)//((-1+x)^2*(1+x^2)),x,5,"atan(x)+log(1-x)"],
[(2+5*x+3*x^2+2*x^3)//(1+x+x^2),x,3,"x+x^2+log(1+x+x^2)"],
[(3-4*x-5*x^2+3*x^3)//(x^3*(-1+x+x^2)),x,5,"3/2/x^2+(-1)/x+3*log(x)-1/10*log(1+2*x-sqrt(5))*(15-sqrt(5))-1/10*log(1+2*x+sqrt(5))*(15+sqrt(5))"],
[(4+8*x+5*x^2+2*x^3)//(2+2*x+x^2)^2,x,5,"(-1)/(2+2*x+x^2)-atan(1+x)+log(2+2*x+x^2)"],
[(-1+x)^4*x^4//(1+x^2),x,3,"4*x-4/3*x^3+x^5-2/3*x^6+1/7*x^7-4*atan(x)"],
[(-20*x+4*x^2)//(9-10*x^2+x^4),x,-11,"log(1-x)-1/2*log(3-x)+3/2*log(1+x)-2*log(3+x)"],
[(-1+x+4*x^3)//((-1+x)*x^2*(1+x^2)),x,5,"(-1)/x+atan(x)+2*log(1-x)-log(1+x^2)"],
[(1-3*x+2*x^2-4*x^3+x^4)//(1+x^2)^3,x,4,"(-1/4)/(1+x^2)^2+2/(1+x^2)+atan(x)"],
[(1-3*x+2*x^2-4*x^3+x^4)//(1+3*x^2+3*x^4+x^6),x,5,"(-1/4)/(1+x^2)^2+2/(1+x^2)+atan(x)"],

#North Texas University Integration Competition
[(1+x+2*x^2+2*x^3)//(x^2+x^3+x^4),x,4,"(-1)/x+log(1+x+x^2)"],
]


for prob in problems
    result = integrate(prob[1], x)
    println("∫", prob[1], "dx = ", result)
    println("expected: ", prob[4])
    println("--------------------------------------------------------------------")
end
